@@ -1,30 +1,17 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-#
-# Copyright (C) 2010-2011 Umeå University
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#            http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 import importlib
 import logging
 
 import re
 from saml2.saml import NAME_FORMAT_URI
-import xmlenc
+import six
+from saml2 import xmlenc
 
 from saml2 import saml
 
 from saml2.time_util import instant, in_a_while
-from saml2.attribute_converter import from_local
+from saml2.attribute_converter import from_local, get_local_name
 from saml2.s_utils import sid, MissingValue
 from saml2.s_utils import factory
 from saml2.s_utils import assertion_factory
@@ -35,25 +22,25 @@ logger = logging.getLogger(__name__)
 
 def _filter_values(vals, vlist=None, must=False):
     """ Removes values from *vals* that does not appear in vlist
-    
+
     :param vals: The values that are to be filtered
     :param vlist: required or optional value
     :param must: Whether the allowed values must appear
     :return: The set of values after filtering
     """
-    
+
     if not vlist:  # No value specified equals any value
         return vals
-    
-    if isinstance(vlist, basestring):
+
+    if isinstance(vlist, six.string_types):
         vlist = [vlist]
-        
+
     res = []
-    
+
     for val in vlist:
         if val in vals:
             res.append(val)
-    
+
     if must:
         if res:
             return res
@@ -78,42 +65,51 @@ def _match(attr, ava):
     return None
 
 
-def filter_on_attributes(ava, required=None, optional=None):
+def filter_on_attributes(ava, required=None, optional=None, acs=None,
+                         fail_on_unfulfilled_requirements=True):
     """ Filter
-    
+
     :param ava: An attribute value assertion as a dictionary
-    :param required: list of RequestedAttribute instances defined to be 
+    :param required: list of RequestedAttribute instances defined to be
         required
     :param optional: list of RequestedAttribute instances defined to be
         optional
+    :param fail_on_unfulfilled_requirements: If required attributes
+        are missing fail or fail not depending on this parameter.
     :return: The modified attribute value assertion
     """
     res = {}
-    
+
     if required is None:
         required = []
 
+    nform = "friendly_name"
     for attr in required:
-        found = False
-        nform = ""
-        for nform in ["friendly_name", "name"]:
-            try:
-                _fn = _match(attr[nform], ava)
-            except KeyError:
-                pass
+        try:
+            _name = attr[nform]
+        except KeyError:
+            if nform == "friendly_name":
+                _name = get_local_name(acs, attr["name"],
+                                       attr["name_format"])
             else:
-                if _fn:
-                    try:
-                        values = [av["text"] for av in attr["attribute_value"]]
-                    except KeyError:
-                        values = []
-                    res[_fn] = _filter_values(ava[_fn], values, True)
-                    found = True
-                    break
+                continue
 
-        if not found:
-            raise MissingValue("Required attribute missing: '%s'" % (
-                attr[nform],))
+        _fn = _match(_name, ava)
+        if not _fn:  # In the unlikely case that someone has provided us
+                     # with URIs as attribute names
+            _fn = _match(attr["name"], ava)
+
+        if _fn:
+            try:
+                values = [av["text"] for av in attr["attribute_value"]]
+            except KeyError:
+                values = []
+            res[_fn] = _filter_values(ava[_fn], values, True)
+            continue
+        elif fail_on_unfulfilled_requirements:
+            desc = "Required attribute missing: '%s' (%s)" % (attr["name"],
+                                                              _name)
+            raise MissingValue(desc)
 
     if optional is None:
         optional = []
@@ -131,20 +127,20 @@ def filter_on_attributes(ava, required=None, optional=None):
                         res[_fn].extend(_filter_values(ava[_fn], values))
                     except KeyError:
                         res[_fn] = _filter_values(ava[_fn], values)
-    
+
     return res
 
 
 def filter_on_demands(ava, required=None, optional=None):
     """ Never return more than is needed. Filters out everything
     the server is prepared to return but the receiver doesn't ask for
-    
+
     :param ava: Attribute value assertion as a dictionary
     :param required: Required attributes
     :param optional: Optional attributes
     :return: The possibly reduced assertion
     """
-    
+
     # Is all what's required there:
     if required is None:
         required = {}
@@ -174,7 +170,7 @@ def filter_on_demands(ava, required=None, optional=None):
     for attr in lava.keys():
         if attr not in oka:
             del ava[lava[attr]]
-    
+
     return ava
 
 
@@ -224,7 +220,7 @@ def filter_attribute_value_assertions(ava, attribute_restrictions=None):
     rules defined in the attribute restrictions. If filtering results in
     an attribute without values, then the attribute is removed from the
     assertion.
-    
+
     :param ava: The incoming attribute value assertion (dictionary)
     :param attribute_restrictions: The rules that govern which attributes
         and values that are allowed. (dictionary)
@@ -232,8 +228,8 @@ def filter_attribute_value_assertions(ava, attribute_restrictions=None):
     """
     if not attribute_restrictions:
         return ava
-    
-    for attr, vals in ava.items():
+
+    for attr, vals in list(ava.items()):
         _attr = attr.lower()
         try:
             _rests = attribute_restrictions[_attr]
@@ -242,7 +238,7 @@ def filter_attribute_value_assertions(ava, attribute_restrictions=None):
         else:
             if _rests is None:
                 continue
-            if isinstance(vals, basestring):
+            if isinstance(vals, six.string_types):
                 vals = [vals]
             rvals = []
             for restr in _rests:
@@ -305,27 +301,28 @@ def post_entity_categories(maps, **kwargs):
 
 class Policy(object):
     """ handles restrictions on assertions """
-    
+
     def __init__(self, restrictions=None):
         if restrictions:
             self.compile(restrictions)
         else:
             self._restrictions = None
-    
+        self.acs = []
+
     def compile(self, restrictions):
         """ This is only for IdPs or AAs, and it's about limiting what
         is returned to the SP.
         In the configuration file, restrictions on which values that
         can be returned are specified with the help of regular expressions.
         This function goes through and pre-compiles the regular expressions.
-        
+
         :param restrictions:
         :return: The assertion with the string specification replaced with
             a compiled regular expression.
         """
-        
+
         self._restrictions = restrictions.copy()
-        
+
         for who, spec in self._restrictions.items():
             if spec is None:
                 continue
@@ -395,7 +392,7 @@ class Policy(object):
             return val
 
     def get_nameid_format(self, sp_entity_id):
-        """ Get the NameIDFormat to used for the entity id 
+        """ Get the NameIDFormat to used for the entity id
         :param: The SP entity ID
         :retur: The format
         """
@@ -403,7 +400,7 @@ class Policy(object):
                         saml.NAMEID_FORMAT_TRANSIENT)
 
     def get_name_form(self, sp_entity_id):
-        """ Get the NameFormat to used for the entity id 
+        """ Get the NameFormat to used for the entity id
         :param: The SP entity ID
         :retur: The format
         """
@@ -411,22 +408,32 @@ class Policy(object):
         return self.get("name_format", sp_entity_id, NAME_FORMAT_URI)
 
     def get_lifetime(self, sp_entity_id):
-        """ The lifetime of the assertion 
+        """ The lifetime of the assertion
         :param sp_entity_id: The SP entity ID
-        :param: lifetime as a dictionary 
+        :param: lifetime as a dictionary
         """
         # default is a hour
         return self.get("lifetime", sp_entity_id, {"hours": 1})
 
     def get_attribute_restrictions(self, sp_entity_id):
         """ Return the attribute restriction for SP that want the information
-        
+
         :param sp_entity_id: The SP entity ID
         :return: The restrictions
         """
 
         return self.get("attribute_restrictions", sp_entity_id)
 
+    def get_fail_on_missing_requested(self, sp_entity_id):
+        """ Return the whether the IdP should should fail if the SPs
+        requested attributes could not be found.
+
+        :param sp_entity_id: The SP entity ID
+        :return: The restrictions
+        """
+
+        return self.get("fail_on_missing_requested", sp_entity_id, True)
+
     def entity_category_attributes(self, ec):
         if not self._restrictions:
             return None
@@ -455,20 +462,20 @@ class Policy(object):
     def not_on_or_after(self, sp_entity_id):
         """ When the assertion stops being valid, should not be
         used after this time.
-        
+
         :param sp_entity_id: The SP entity ID
         :return: String representation of the time
         """
-        
+
         return in_a_while(**self.get_lifetime(sp_entity_id))
-    
+
     def filter(self, ava, sp_entity_id, mdstore, required=None, optional=None):
         """ What attribute and attribute values returns depends on what
         the SP has said it wants in the request or in the metadata file and
         what the IdP/AA wants to release. An assumption is that what the SP
         asks for overrides whatever is in the metadata. But of course the
         IdP never releases anything it doesn't want to.
-        
+
         :param ava: The information about the subject as a dictionary
         :param sp_entity_id: The entity ID of the SP
         :param mdstore: A Metadata store
@@ -477,21 +484,38 @@ class Policy(object):
         :return: A possibly modified AVA
         """
 
-        _rest = self.get_attribute_restrictions(sp_entity_id)
-        if _rest is None:
-            _rest = self.get_entity_categories(sp_entity_id, mdstore)
-        logger.debug("filter based on: %s" % _rest)
-        ava = filter_attribute_value_assertions(ava, _rest)
-        
+        _ava = None
         if required or optional:
-            ava = filter_on_attributes(ava, required, optional)
-        
-        return ava
-    
+            logger.debug("required: %s, optional: %s" % (required, optional))
+            _ava = filter_on_attributes(
+                ava.copy(), required, optional, self.acs,
+                self.get_fail_on_missing_requested(sp_entity_id))
+
+        _rest = self.get_entity_categories(sp_entity_id, mdstore)
+        if _rest:
+            ava_ec = filter_attribute_value_assertions(ava.copy(), _rest)
+            if _ava is None:
+                _ava = ava_ec
+            else:
+                _ava.update(ava_ec)
+
+        _rest = self.get_attribute_restrictions(sp_entity_id)
+        if _rest:
+            if _ava is None:
+                _ava = ava.copy()
+            _ava = filter_attribute_value_assertions(_ava, _rest)
+        elif _ava is None:
+            _ava = ava.copy()
+
+        if _ava is None:
+            return {}
+        else:
+            return _ava
+
     def restrict(self, ava, sp_entity_id, metadata=None):
         """ Identity attribute names are expected to be expressed in
         the local lingo (== friendlyName)
-        
+
         :return: A filtered ava according to the IdPs/AAs rules and
             the list of required/optional attributes according to the SP.
             If the requirements can't be met an exception is raised.
@@ -499,14 +523,14 @@ class Policy(object):
         if metadata:
             spec = metadata.attribute_requirement(sp_entity_id)
             if spec:
-                ava = self.filter(ava, sp_entity_id, metadata,
-                                  spec["required"], spec["optional"])
+                return self.filter(ava, sp_entity_id, metadata,
+                                   spec["required"], spec["optional"])
 
         return self.filter(ava, sp_entity_id, metadata, [], [])
 
     def conditions(self, sp_entity_id):
         """ Return a saml.Condition instance
-        
+
         :param sp_entity_id: The SP entity ID
         :return: A saml.Condition instance
         """
@@ -535,115 +559,119 @@ class EntityCategories(object):
     pass
 
 
+def _authn_context_class_ref(authn_class, authn_auth=None):
+    """
+    Construct the authn context with a authn context class reference
+    :param authn_class: The authn context class reference
+    :param authn_auth: Authenticating Authority
+    :return: An AuthnContext instance
+    """
+    cntx_class = factory(saml.AuthnContextClassRef, text=authn_class)
+    if authn_auth:
+        return factory(saml.AuthnContext,
+                       authn_context_class_ref=cntx_class,
+                       authenticating_authority=factory(
+                           saml.AuthenticatingAuthority, text=authn_auth))
+    else:
+        return factory(saml.AuthnContext,
+                       authn_context_class_ref=cntx_class)
+
+
+def _authn_context_decl(decl, authn_auth=None):
+    """
+    Construct the authn context with a authn context declaration
+    :param decl: The authn context declaration
+    :param authn_auth: Authenticating Authority
+    :return: An AuthnContext instance
+    """
+    return factory(saml.AuthnContext,
+                   authn_context_decl=decl,
+                   authenticating_authority=factory(
+                       saml.AuthenticatingAuthority, text=authn_auth))
+
+
+def _authn_context_decl_ref(decl_ref, authn_auth=None):
+    """
+    Construct the authn context with a authn context declaration reference
+    :param decl_ref: The authn context declaration reference
+    :param authn_auth: Authenticating Authority
+    :return: An AuthnContext instance
+    """
+    return factory(saml.AuthnContext,
+                   authn_context_decl_ref=decl_ref,
+                   authenticating_authority=factory(
+                       saml.AuthenticatingAuthority, text=authn_auth))
+
+
+def authn_statement(authn_class=None, authn_auth=None,
+                    authn_decl=None, authn_decl_ref=None, authn_instant="",
+                    subject_locality=""):
+    """
+    Construct the AuthnStatement
+    :param authn_class: Authentication Context Class reference
+    :param authn_auth: Authenticating Authority
+    :param authn_decl: Authentication Context Declaration
+    :param authn_decl_ref: Authentication Context Declaration reference
+    :param authn_instant: When the Authentication was performed.
+        Assumed to be seconds since the Epoch.
+    :param subject_locality: Specifies the DNS domain name and IP address
+        for the system from which the assertion subject was apparently
+        authenticated.
+    :return: An AuthnContext instance
+    """
+    if authn_instant:
+        _instant = instant(time_stamp=authn_instant)
+    else:
+        _instant = instant()
+
+    if authn_class:
+        res = factory(
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            authn_context=_authn_context_class_ref(
+                authn_class, authn_auth))
+    elif authn_decl:
+        res = factory(
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            authn_context=_authn_context_decl(authn_decl, authn_auth))
+    elif authn_decl_ref:
+        res = factory(
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid(),
+            authn_context=_authn_context_decl_ref(authn_decl_ref,
+                                                       authn_auth))
+    else:
+        res = factory(
+            saml.AuthnStatement,
+            authn_instant=_instant,
+            session_index=sid())
+
+    if subject_locality:
+        res.subject_locality = saml.SubjectLocality(text=subject_locality)
+
+    return res
+
+
 class Assertion(dict):
     """ Handles assertions about subjects """
-    
+
     def __init__(self, dic=None):
         dict.__init__(self, dic)
-    
-    def _authn_context_decl(self, decl, authn_auth=None):
-        """
-        Construct the authn context with a authn context declaration
-        :param decl: The authn context declaration
-        :param authn_auth: Authenticating Authority
-        :return: An AuthnContext instance
-        """
-        return factory(saml.AuthnContext,
-                       authn_context_decl=decl,
-                       authenticating_authority=factory(
-                           saml.AuthenticatingAuthority, text=authn_auth))
-
-    def _authn_context_decl_ref(self, decl_ref, authn_auth=None):
-        """
-        Construct the authn context with a authn context declaration reference
-        :param decl_ref: The authn context declaration reference
-        :param authn_auth: Authenticating Authority
-        :return: An AuthnContext instance
-        """
-        return factory(saml.AuthnContext,
-                       authn_context_decl_ref=decl_ref,
-                       authenticating_authority=factory(
-                           saml.AuthenticatingAuthority, text=authn_auth))
-
-    @staticmethod
-    def _authn_context_class_ref(authn_class, authn_auth=None):
-        """
-        Construct the authn context with a authn context class reference
-        :param authn_class: The authn context class reference
-        :param authn_auth: Authenticating Authority
-        :return: An AuthnContext instance
-        """
-        cntx_class = factory(saml.AuthnContextClassRef, text=authn_class)
-        if authn_auth:
-            return factory(saml.AuthnContext, 
-                           authn_context_class_ref=cntx_class,
-                           authenticating_authority=factory(
-                               saml.AuthenticatingAuthority, text=authn_auth))
-        else:
-            return factory(saml.AuthnContext,
-                           authn_context_class_ref=cntx_class)
-        
-    def _authn_statement(self, authn_class=None, authn_auth=None,
-                         authn_decl=None, authn_decl_ref=None, authn_instant="",
-                         subject_locality=""):
-        """
-        Construct the AuthnStatement
-        :param authn_class: Authentication Context Class reference
-        :param authn_auth: Authenticating Authority
-        :param authn_decl: Authentication Context Declaration
-        :param authn_decl_ref: Authentication Context Declaration reference
-        :param authn_instant: When the Authentication was performed.
-            Assumed to be seconds since the Epoch.
-        :param subject_locality: Specifies the DNS domain name and IP address
-            for the system from which the assertion subject was apparently
-            authenticated.
-        :return: An AuthnContext instance
-        """
-        if authn_instant:
-            _instant = instant(time_stamp=authn_instant)
-        else:
-            _instant = instant()
-
-        if authn_class:
-            res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid(),
-                authn_context=self._authn_context_class_ref(
-                    authn_class, authn_auth))
-        elif authn_decl:
-            res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid(),
-                authn_context=self._authn_context_decl(authn_decl, authn_auth))
-        elif authn_decl_ref:
-            res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid(),
-                authn_context=self._authn_context_decl_ref(authn_decl_ref,
-                                                           authn_auth))
-        else:
-            res = factory(
-                saml.AuthnStatement,
-                authn_instant=_instant,
-                session_index=sid())
-
-        if subject_locality:
-            res.subject_locality = saml.SubjectLocality(text=subject_locality)
-
-        return res
+        self.acs = []
 
     def construct(self, sp_entity_id, in_response_to, consumer_url,
                   name_id, attrconvs, policy, issuer, authn_class=None,
                   authn_auth=None, authn_decl=None, encrypt=None,
                   sec_context=None, authn_decl_ref=None, authn_instant="",
-                  subject_locality=""):
-        """ Construct the Assertion 
-        
+                  subject_locality="", authn_statem=None, add_subject=True):
+        """ Construct the Assertion
+
         :param sp_entity_id: The entityid of the SP
-        :param in_response_to: An identifier of the message, this message is 
+        :param in_response_to: An identifier of the message, this message is
             a response to
         :param consumer_url: The intended consumer of the assertion
         :param name_id: An NameID instance
@@ -660,6 +688,7 @@ class Assertion(dict):
         :param subject_locality: Specifies the DNS domain name and IP address
             for the system from which the assertion subject was apparently
             authenticated.
+        :param authn_statem: A AuthnStatement instance
         :return: An Assertion instance
         """
 
@@ -684,31 +713,39 @@ class Assertion(dict):
         # start using now and for some time
         conds = policy.conditions(sp_entity_id)
 
-        if authn_auth or authn_class or authn_decl or authn_decl_ref:
-            _authn_statement = self._authn_statement(authn_class, authn_auth,
-                                                     authn_decl, authn_decl_ref,
-                                                     authn_instant,
-                                                     subject_locality)
+        if authn_statem:
+            _authn_statement = authn_statem
+        elif authn_auth or authn_class or authn_decl or authn_decl_ref:
+            _authn_statement = authn_statement(authn_class, authn_auth,
+                                               authn_decl, authn_decl_ref,
+                                               authn_instant,
+                                               subject_locality)
         else:
             _authn_statement = None
 
-
-        _ass = assertion_factory(
-            issuer=issuer,
-            conditions=conds,
-            subject=factory(
-                saml.Subject,
-                name_id=name_id,
-                subject_confirmation=[factory(
-                    saml.SubjectConfirmation,
-                    method=saml.SCM_BEARER,
-                    subject_confirmation_data=factory(
-                        saml.SubjectConfirmationData,
-                        in_response_to=in_response_to,
-                        recipient=consumer_url,
-                        not_on_or_after=policy.not_on_or_after(sp_entity_id)))]
-            ),
-        )
+        if not add_subject:
+            _ass = assertion_factory(
+                issuer=issuer,
+                conditions=conds,
+                subject=None
+            )
+        else:
+            _ass = assertion_factory(
+                issuer=issuer,
+                conditions=conds,
+                subject=factory(
+                    saml.Subject,
+                    name_id=name_id,
+                    subject_confirmation=[factory(
+                        saml.SubjectConfirmation,
+                        method=saml.SCM_BEARER,
+                        subject_confirmation_data=factory(
+                            saml.SubjectConfirmationData,
+                            in_response_to=in_response_to,
+                            recipient=consumer_url,
+                            not_on_or_after=policy.not_on_or_after(sp_entity_id)))]
+                ),
+            )
 
         if _authn_statement:
             _ass.authn_statement = [_authn_statement]
@@ -717,15 +754,23 @@ class Assertion(dict):
             _ass.attribute_statement=[attr_statement]
 
         return _ass
-    
+
     def apply_policy(self, sp_entity_id, policy, metadata=None):
-        """ Apply policy to the assertion I'm representing 
-        
+        """ Apply policy to the assertion I'm representing
+
         :param sp_entity_id: The SP entity ID
         :param policy: The policy
         :param metadata: Metadata to use
         :return: The resulting AVA after the policy is applied
         """
+
+        policy.acs = self.acs
         ava = policy.restrict(self, sp_entity_id, metadata)
-        self.update(ava)
-        return ava
\ No newline at end of file
+
+        for key, val in list(self.items()):
+            if key in ava:
+                self[key] = ava[key]
+            else:
+                del self[key]
+
+        return ava
