@@ -1,22 +1,10 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010-2011 Umeå University
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#            http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 
 import calendar
 import logging
+import six
 from saml2.samlp import STATUS_VERSION_MISMATCH
 from saml2.samlp import STATUS_AUTHN_FAILED
 from saml2.samlp import STATUS_INVALID_ATTR_NAME_OR_VALUE
@@ -37,13 +25,14 @@ from saml2.samlp import STATUS_TOO_MANY_RESPONSES
 from saml2.samlp import STATUS_UNKNOWN_ATTR_PROFILE
 from saml2.samlp import STATUS_UNKNOWN_PRINCIPAL
 from saml2.samlp import STATUS_UNSUPPORTED_BINDING
+from saml2.samlp import STATUS_RESPONDER
 
-import xmldsig as ds
-import xmlenc as xenc
+from saml2 import xmldsig as ds
+from saml2 import xmlenc as xenc
 
 from saml2 import samlp
+from saml2 import class_name
 from saml2 import saml
-from saml2 import extension_element_to_element
 from saml2 import extension_elements_to_elements
 from saml2 import SAMLError
 from saml2 import time_util
@@ -171,6 +160,8 @@ class StatusUnknownPrincipal(StatusError):
 class StatusUnsupportedBinding(StatusError):
     pass
 
+class StatusResponder(StatusError):
+    pass
 
 STATUSCODE2EXCEPTION = {
     STATUS_VERSION_MISMATCH: StatusVersionMismatch,
@@ -193,6 +184,7 @@ STATUSCODE2EXCEPTION = {
     STATUS_UNKNOWN_ATTR_PROFILE: StatusUnknownAttrProfile,
     STATUS_UNKNOWN_PRINCIPAL: StatusUnknownPrincipal,
     STATUS_UNSUPPORTED_BINDING: StatusUnsupportedBinding,
+    STATUS_RESPONDER: StatusResponder,
 }
 # ---------------------------------------------------------------------------
 
@@ -204,7 +196,7 @@ def _dummy(_):
 def for_me(conditions, myself):
     """ Am I among the intended audiences """
 
-    if not conditions.audience_restriction: # No audience restriction
+    if not conditions.audience_restriction:  # No audience restriction
         return True
 
     for restriction in conditions.audience_restriction:
@@ -214,21 +206,22 @@ def for_me(conditions, myself):
             if audience.text.strip() == myself:
                 return True
             else:
-                #print "Not for me: %s != %s" % (audience.text.strip(), myself)
+                #print("Not for me: %s != %s" % (audience.text.strip(), myself))
                 pass
-    
+
     return False
 
 
 def authn_response(conf, return_addrs, outstanding_queries=None, timeslack=0,
-                   asynchop=True, allow_unsolicited=False, want_assertions_signed=False):
+                   asynchop=True, allow_unsolicited=False,
+                   want_assertions_signed=False):
     sec = security_context(conf)
     if not timeslack:
         try:
             timeslack = int(conf.accepted_time_diff)
         except TypeError:
             timeslack = 0
-    
+
     return AuthnResponse(sec, conf.attribute_converters, conf.entityid,
                          return_addrs, outstanding_queries, timeslack,
                          asynchop=asynchop, allow_unsolicited=allow_unsolicited,
@@ -262,6 +255,7 @@ class StatusResponse(object):
         self.request_id = request_id
 
         self.xmlstr = ""
+        self.origxml = ""
         self.name_id = None
         self.response = None
         self.not_on_or_after = 0
@@ -271,13 +265,13 @@ class StatusResponse(object):
         self.require_response_signature = False
         self.not_signed = False
         self.asynchop = asynchop
-    
+
     def _clear(self):
         self.xmlstr = ""
         self.name_id = None
         self.response = None
         self.not_on_or_after = 0
-        
+
     def _postamble(self):
         if not self.response:
             logger.error("Response was not correctly signed")
@@ -289,14 +283,14 @@ class StatusResponse(object):
 
         try:
             valid_instance(self.response)
-        except NotValid, exc:
+        except NotValid as exc:
             logger.error("Not valid response: %s" % exc.args[0])
             self._clear()
             return self
-        
+
         self.in_response_to = self.response.in_response_to
         return self
-        
+
     def load_instance(self, instance):
         if signed(instance):
             # This will check signature on Assertion which is the default
@@ -309,31 +303,36 @@ class StatusResponse(object):
         else:
             self.not_signed = True
             self.response = instance
-            
+
         return self._postamble()
-        
+
     def _loads(self, xmldata, decode=True, origxml=None):
 
         # own copy
         self.xmlstr = xmldata[:]
         logger.debug("xmlstr: %s" % (self.xmlstr,))
+        if origxml:
+            self.origxml = origxml
+        else:
+            self.origxml = self.xmlstr
 
         try:
-            self.response = self.signature_check(xmldata, origdoc=origxml, must=self.require_signature,
-                                                 require_response_signature=self.require_response_signature)
+            self.response = self.signature_check(
+                xmldata, origdoc=origxml, must=self.require_signature,
+                require_response_signature=self.require_response_signature)
 
         except TypeError:
             raise
         except SignatureError:
             raise
-        except Exception, excp:
-            #logger.exception("EXCEPTION: %s", excp)
+        except Exception as excp:
+            logger.exception("EXCEPTION: %s", excp)
             raise
-    
-        #print "<", self.response
-        
+
+        #print("<", self.response)
+
         return self._postamble()
-    
+
     def status_ok(self):
         if self.response.status:
             status = self.response.status
@@ -362,14 +361,14 @@ class StatusResponse(object):
                                      self.timeslack).timetuple()
         lower = time_util.shift_time(time_util.time_a_while_ago(days=1),
                                      -self.timeslack).timetuple()
-        # print "issue_instant: %s" % self.response.issue_instant
-        # print "%s < x < %s" % (lower, upper)
+        # print("issue_instant: %s" % self.response.issue_instant)
+        # print("%s < x < %s" % (lower, upper))
         issued_at = str_to_time(self.response.issue_instant)
         return lower < issued_at < upper
 
     def _verify(self):
         if self.request_id and self.in_response_to and \
-                self.in_response_to != self.request_id:
+                        self.in_response_to != self.request_id:
             logger.error("Not the id I expected: %s != %s" % (
                 self.in_response_to, self.request_id))
             return None
@@ -387,9 +386,9 @@ class StatusResponse(object):
             if self.response.destination and \
                     self.response.destination not in self.return_addrs:
                 logger.error("%s not in %s" % (self.response.destination,
-                                           self.return_addrs))
+                                               self.return_addrs))
                 return None
-            
+
         assert self.issue_instant_ok()
         assert self.status_ok()
         return self
@@ -397,7 +396,7 @@ class StatusResponse(object):
     def loads(self, xmldata, decode=True, origxml=None):
         return self._loads(xmldata, decode, origxml)
 
-    def verify(self):
+    def verify(self, keys=None):
         try:
             return self._verify()
         except AssertionError:
@@ -408,10 +407,10 @@ class StatusResponse(object):
         self.xmlstr = mold.xmlstr
         self.in_response_to = mold.in_response_to
         self.response = mold.response
-        
+
     def issuer(self):
         return self.response.issuer.text.strip()
-        
+
 
 class LogoutResponse(StatusResponse):
     msgtype = "logout_response"
@@ -430,7 +429,8 @@ class NameIDMappingResponse(StatusResponse):
                  request_id=0, asynchop=True):
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
                                 request_id, asynchop)
-        self.signature_check = self.sec.correctly_signed_name_id_mapping_response
+        self.signature_check = self.sec\
+            .correctly_signed_name_id_mapping_response
 
 
 class ManageNameIDResponse(StatusResponse):
@@ -455,7 +455,8 @@ class AuthnResponse(StatusResponse):
                  return_addrs=None, outstanding_queries=None,
                  timeslack=0, asynchop=True, allow_unsolicited=False,
                  test=False, allow_unknown_attributes=False,
-                 want_assertions_signed=False, want_response_signed=False, **kwargs):
+                 want_assertions_signed=False, want_response_signed=False,
+                 **kwargs):
 
         StatusResponse.__init__(self, sec_context, return_addrs, timeslack,
                                 asynchop=asynchop)
@@ -465,10 +466,11 @@ class AuthnResponse(StatusResponse):
             self.outstanding_queries = outstanding_queries
         else:
             self.outstanding_queries = {}
-        self.context = "AuthnReq"        
-        self.came_from = ""
+        self.context = "AuthnReq"
+        self.came_from = None
         self.ava = None
         self.assertion = None
+        self.assertions = []
         self.session_not_on_or_after = 0
         self.allow_unsolicited = allow_unsolicited
         self.require_signature = want_assertions_signed
@@ -481,27 +483,48 @@ class AuthnResponse(StatusResponse):
         except KeyError:
             self.extension_schema = {}
 
+    def check_subject_confirmation_in_response_to(self, irp):
+        for assertion in self.response.assertion:
+            for _sc in assertion.subject.subject_confirmation:
+                try:
+                    assert _sc.subject_confirmation_data.in_response_to == irp
+                except AssertionError:
+                    return False
+
+        return True
+
     def loads(self, xmldata, decode=True, origxml=None):
         self._loads(xmldata, decode, origxml)
-        
+
         if self.asynchop:
             if self.in_response_to in self.outstanding_queries:
                 self.came_from = self.outstanding_queries[self.in_response_to]
-                del self.outstanding_queries[self.in_response_to]
+                #del self.outstanding_queries[self.in_response_to]
+                try:
+                    if not self.check_subject_confirmation_in_response_to(
+                            self.in_response_to):
+                        logger.exception(
+                            "Unsolicited response %s" % self.in_response_to)
+                        raise UnsolicitedResponse(
+                            "Unsolicited response: %s" % self.in_response_to)
+                except AttributeError:
+                    pass
             elif self.allow_unsolicited:
                 pass
             else:
-                logger.exception("Unsolicited response %s" % self.in_response_to)
-                raise UnsolicitedResponse("Unsolicited response: %s" % self.in_response_to)
-            
+                logger.exception(
+                    "Unsolicited response %s" % self.in_response_to)
+                raise UnsolicitedResponse(
+                    "Unsolicited response: %s" % self.in_response_to)
+
         return self
 
     def clear(self):
         self._clear()
-        self.came_from = ""
+        self.came_from = None
         self.ava = None
         self.assertion = None
-        
+
     def authn_statement_ok(self, optional=False):
         try:
             # the assertion MUST contain one AuthNStatement
@@ -510,8 +533,9 @@ class AuthnResponse(StatusResponse):
             if optional:
                 return True
             else:
+                logger.error("No AuthnStatement")
                 raise
-            
+
         authn_statement = self.assertion.authn_statement[0]
         if authn_statement.session_not_on_or_after:
             if validate_on_or_after(authn_statement.session_not_on_or_after,
@@ -523,7 +547,7 @@ class AuthnResponse(StatusResponse):
                 return False
         return True
         # check authn_statement.session_index
-    
+
     def condition_ok(self, lax=False):
         if self.test:
             lax = True
@@ -541,7 +565,8 @@ class AuthnResponse(StatusResponse):
 
         # if both are present NotBefore must be earlier than NotOnOrAfter
         if conditions.not_before and conditions.not_on_or_after:
-            if not later_than(conditions.not_on_or_after, conditions.not_before):
+            if not later_than(conditions.not_on_or_after,
+                              conditions.not_before):
                 return False
 
         try:
@@ -550,7 +575,7 @@ class AuthnResponse(StatusResponse):
                     conditions.not_on_or_after, self.timeslack)
             if conditions.not_before:
                 validate_before(conditions.not_before, self.timeslack)
-        except Exception, excp:
+        except Exception as excp:
             logger.error("Exception on conditions: %s" % (excp,))
             if not lax:
                 raise
@@ -562,10 +587,11 @@ class AuthnResponse(StatusResponse):
                 if not lax:
                     raise Exception("Not for me!!!")
 
-        if conditions.condition: # extra conditions
+        if conditions.condition:  # extra conditions
             for cond in conditions.condition:
                 try:
-                    if cond.extension_attributes[XSI_TYPE] in self.extension_schema:
+                    if cond.extension_attributes[
+                        XSI_TYPE] in self.extension_schema:
                         pass
                     else:
                         raise Exception("Unknown condition")
@@ -582,9 +608,9 @@ class AuthnResponse(StatusResponse):
         :param attribute_statement: A SAML.AttributeStatement which might
             contain both encrypted attributes and attributes.
         """
-#        _node_name = [
-#            "urn:oasis:names:tc:SAML:2.0:assertion:EncryptedData",
-#            "urn:oasis:names:tc:SAML:2.0:assertion:EncryptedAttribute"]
+        #        _node_name = [
+        #            "urn:oasis:names:tc:SAML:2.0:assertion:EncryptedData",
+        #            "urn:oasis:names:tc:SAML:2.0:assertion:EncryptedAttribute"]
 
         for encattr in attribute_statement.encrypted_attribute:
             if not encattr.encrypted_key:
@@ -597,24 +623,33 @@ class AuthnResponse(StatusResponse):
                 attrlist = enc_attr.extensions_as_elements("Attribute", saml)
                 attribute_statement.attribute.extend(attrlist)
 
+    def read_attribute_statement(self, attr_statem):
+        logger.debug("Attribute Statement: %s" % (attr_statem,))
+        for aconv in self.attribute_converters:
+            logger.debug("Converts name format: %s" % (aconv.name_format,))
+
+        self.decrypt_attributes(attr_statem)
+        return to_local(self.attribute_converters, attr_statem,
+                       self.allow_unknown_attributes)
+
     def get_identity(self):
         """ The assertion can contain zero or one attributeStatements
 
         """
-        if not self.assertion.attribute_statement:
-            logger.error("Missing Attribute Statement")
-            ava = {}
-        else:
-            assert len(self.assertion.attribute_statement) == 1
-            _attr_statem = self.assertion.attribute_statement[0]
-
-            logger.debug("Attribute Statement: %s" % (_attr_statem,))
-            for aconv in self.attribute_converters:
-                logger.debug("Converts name format: %s" % (aconv.name_format,))
-
-            self.decrypt_attributes(_attr_statem)
-            ava = to_local(self.attribute_converters, _attr_statem,
-                           self.allow_unknown_attributes)
+        ava = {}
+        for _assertion in self.assertions:
+            if _assertion.advice:
+                if _assertion.advice.assertion:
+                    for tmp_assertion in _assertion.advice.assertion:
+                        if tmp_assertion.attribute_statement:
+                            assert len(tmp_assertion.attribute_statement) == 1
+                            ava.update(self.read_attribute_statement(tmp_assertion.attribute_statement[0]))
+            if _assertion.attribute_statement:
+                assert len(_assertion.attribute_statement) == 1
+                _attr_statem = _assertion.attribute_statement[0]
+                ava.update(self.read_attribute_statement(_attr_statem))
+            if not ava:
+                logger.error("Missing Attribute Statement")
         return ava
 
     def _bearer_confirmed(self, data):
@@ -624,7 +659,7 @@ class AuthnResponse(StatusResponse):
         if data.address:
             if not valid_address(data.address):
                 return False
-            # verify that I got it from the correct sender
+                # verify that I got it from the correct sender
 
         # These two will raise exception if untrue
         validate_on_or_after(data.not_on_or_after, self.timeslack)
@@ -634,12 +669,12 @@ class AuthnResponse(StatusResponse):
         if not later_than(data.not_on_or_after, data.not_before):
             return False
 
-        if self.asynchop and not self.came_from:
+        if self.asynchop and self.came_from is None:
             if data.in_response_to:
                 if data.in_response_to in self.outstanding_queries:
                     self.came_from = self.outstanding_queries[
                         data.in_response_to]
-                    del self.outstanding_queries[data.in_response_to]
+                    #del self.outstanding_queries[data.in_response_to]
                 elif self.allow_unsolicited:
                     pass
                 else:
@@ -650,7 +685,8 @@ class AuthnResponse(StatusResponse):
                     logger.info("outstanding queries: %s" % (
                         self.outstanding_queries.keys(),))
                     raise Exception(
-                        "Combination of session id and requestURI I don't recall")
+                        "Combination of session id and requestURI I don't "
+                        "recall")
         return True
 
     def _holder_of_key_confirmed(self, data):
@@ -687,12 +723,12 @@ class AuthnResponse(StatusResponse):
                     subject_confirmation.method,))
 
             subjconf.append(subject_confirmation)
-            
+
         if not subjconf:
             raise VerificationError("No valid subject confirmation")
-            
+
         subject.subject_confirmation = subjconf
-        
+
         # The subject must contain a name_id
         try:
             assert subject.name_id
@@ -709,64 +745,132 @@ class AuthnResponse(StatusResponse):
 
         logger.info("Subject NameID: %s" % self.name_id)
         return self.name_id
-    
-    def _assertion(self, assertion):
-        self.assertion = assertion
 
+    def _assertion(self, assertion, verified=False):
+        """
+        Check the assertion
+        :param assertion:
+        :return: True/False depending on if the assertion is sane or not
+        """
+
+        if not hasattr(assertion, 'signature') or not assertion.signature:
+            logger.debug("unsigned")
+            if self.require_signature:
+                raise SignatureError("Signature missing for assertion")
+        else:
+            logger.debug("signed")
+            if not verified:
+                try:
+                    self.sec.check_signature(assertion, class_name(assertion),self.xmlstr)
+                except Exception as exc:
+                    logger.error("correctly_signed_response: %s" % exc)
+                    raise
+
+        self.assertion = assertion
         logger.debug("assertion context: %s" % (self.context,))
         logger.debug("assertion keys: %s" % (assertion.keyswv()))
         logger.debug("outstanding_queries: %s" % (self.outstanding_queries,))
-        
+
         #if self.context == "AuthnReq" or self.context == "AttrQuery":
         if self.context == "AuthnReq":
             self.authn_statement_ok()
-#        elif self.context == "AttrQuery":
-#            self.authn_statement_ok(True)
+        #        elif self.context == "AttrQuery":
+        #            self.authn_statement_ok(True)
 
         if not self.condition_ok():
             raise VerificationError("Condition not OK")
 
         logger.debug("--- Getting Identity ---")
 
-        if self.context == "AuthnReq" or self.context == "AttrQuery":
-            self.ava = self.get_identity()
+        #if self.context == "AuthnReq" or self.context == "AttrQuery":
+        #    self.ava = self.get_identity()
+        #    logger.debug("--- AVA: %s" % (self.ava,))
 
-            logger.debug("--- AVA: %s" % (self.ava,))
-        
         try:
             self.get_subject()
             if self.asynchop:
                 if self.allow_unsolicited:
                     pass
-                elif not self.came_from:
+                elif self.came_from is None:
                     raise VerificationError("Came from")
             return True
         except Exception:
             logger.exception("get subject")
             raise
-    
-    def _encrypted_assertion(self, xmlstr):
-        if xmlstr.encrypted_data:
-            assertion_str = self.sec.decrypt(xmlstr.encrypted_data.to_string())
-            if not assertion_str:
-                raise DecryptionFailed()
-            assertion = saml.assertion_from_string(assertion_str)
-        else:
-            decrypt_xml = self.sec.decrypt(xmlstr)
 
-            logger.debug("Decryption successfull")
+    def decrypt_assertions(self, encrypted_assertions, decr_txt, issuer=None, verified=False):
+        """ Moves the decrypted assertion from the encrypted assertion to a list.
 
-            self.response = samlp.response_from_string(decrypt_xml)
-            logger.debug("Parsed decrypted assertion successfull")
+        :param encrypted_assertions: A list of encrypted assertions.
+        :param decr_txt: The string representation containing the decrypted data. Used when verifying signatures.
+        :param issuer: The issuer of the response.
+        :param verified: If True do not verify signatures, otherwise verify the signature if it exists.
+        :return: A list of decrypted assertions.
+        """
+        res = []
+        for encrypted_assertion in encrypted_assertions:
+            if encrypted_assertion.extension_elements:
+                assertions = extension_elements_to_elements(
+                    encrypted_assertion.extension_elements, [saml, samlp])
+                for assertion in assertions:
+                    if assertion.signature and not verified:
+                        if not self.sec.check_signature(
+                                assertion, origdoc=decr_txt,
+                                node_name=class_name(assertion), issuer=issuer):
+                            logger.error("Failed to verify signature on '%s'" % assertion)
+                            raise SignatureError()
+                    res.append(assertion)
+        return res
 
-            enc = self.response.encrypted_assertion[0].extension_elements[0]
-            assertion = extension_element_to_element(
-                enc, saml.ELEMENT_FROM_STRING, namespace=saml.NAMESPACE)
+    def find_encrypt_data_assertion(self, enc_assertions):
+        """ Verifies if a list of encrypted assertions contains encrypted data.
 
-        logger.debug("Decrypted Assertion: %s" % assertion)
-        return self._assertion(assertion)
-    
-    def parse_assertion(self):
+        :param enc_assertions: A list of encrypted assertions.
+        :return: True encrypted data exists otherwise false.
+        """
+        for _assertion in enc_assertions:
+                if _assertion.encrypted_data is not None:
+                    return True
+
+    def find_encrypt_data_assertion_list(self, _assertions):
+        """ Verifies if a list of assertions contains encrypted data in the advice element.
+
+        :param _assertions: A list of assertions.
+        :return: True encrypted data exists otherwise false.
+        """
+        for _assertion in _assertions:
+            if _assertion.advice:
+                if _assertion.advice.encrypted_assertion:
+                    res = self.find_encrypt_data_assertion(_assertion.advice.encrypted_assertion)
+                    if res:
+                        return True
+
+    def find_encrypt_data(self, resp):
+        """ Verifies if a saml response contains encrypted assertions with encrypted data.
+
+        :param resp: A saml response.
+        :return: True encrypted data exists otherwise false.
+        """
+        _has_encrypt_data = False
+        if resp.encrypted_assertion:
+            res = self.find_encrypt_data_assertion(resp.encrypted_assertion)
+            if res:
+                return True
+        if resp.assertion:
+            for tmp_assertion in resp.assertion:
+                if tmp_assertion.advice:
+                    if tmp_assertion.advice.encrypted_assertion:
+                        res = self.find_encrypt_data_assertion(tmp_assertion.advice.encrypted_assertion)
+                        if res:
+                            return True
+        return False
+
+    def parse_assertion(self, keys=None):
+        """ Parse the assertions for a saml response.
+
+        :param keys: A string representing a RSA key or a list of strings containing RSA keys.
+        :return: True if the assertions are parsed otherwise False.
+        """
         if self.context == "AuthnQuery":
             # can contain one or more assertions
             pass
@@ -776,46 +880,112 @@ class AuthnResponse(StatusResponse):
                     len(self.response.encrypted_assertion) == 1
             except AssertionError:
                 raise Exception("No assertion part")
-        
-        if self.response.assertion:
-            logger.debug("***Unencrypted response***")
-            for assertion in self.response.assertion:
-                if not self._assertion(assertion):
-                    return False
-            return True
-        else:
-            logger.debug("***Encrypted response***")
-            for assertion in self.response.encrypted_assertion:
-                if not self._encrypted_assertion(assertion):
-                    return False
-            return True
 
-    def verify(self):
+        has_encrypted_assertions = self.find_encrypt_data(self.response) #self.response.encrypted_assertion
+        #if not has_encrypted_assertions and self.response.assertion:
+        #    for tmp_assertion in self.response.assertion:
+        #        if tmp_assertion.advice:
+        #            if tmp_assertion.advice.encrypted_assertion:
+        #                has_encrypted_assertions = True
+        #                break
+
+        if self.response.assertion:
+            logger.debug("***Unencrypted assertion***")
+            for assertion in self.response.assertion:
+                if not self._assertion(assertion, False):
+                    return False
+
+        if has_encrypted_assertions:
+            _enc_assertions = []
+            logger.debug("***Encrypted assertion/-s***")
+            decr_text = "%s" % self.response
+            resp = self.response
+            decr_text_old = None
+            while self.find_encrypt_data(resp) and decr_text_old != decr_text:
+                decr_text_old = decr_text
+                decr_text = self.sec.decrypt_keys(decr_text, keys)
+                resp = samlp.response_from_string(decr_text)
+            _enc_assertions = self.decrypt_assertions(resp.encrypted_assertion, decr_text)
+            decr_text_old = None
+            while (self.find_encrypt_data(resp) or self.find_encrypt_data_assertion_list(_enc_assertions)) and \
+                            decr_text_old != decr_text:
+                decr_text_old = decr_text
+                decr_text = self.sec.decrypt_keys(decr_text, keys)
+                resp = samlp.response_from_string(decr_text)
+                _enc_assertions = self.decrypt_assertions(resp.encrypted_assertion, decr_text, verified=True)
+            #_enc_assertions = self.decrypt_assertions(resp.encrypted_assertion, decr_text, verified=True)
+            all_assertions = _enc_assertions
+            if resp.assertion:
+                all_assertions = all_assertions + resp.assertion
+            if len(all_assertions) > 0:
+                for tmp_ass in all_assertions:
+                    if tmp_ass.advice and tmp_ass.advice.encrypted_assertion:
+
+                        advice_res = self.decrypt_assertions(tmp_ass.advice.encrypted_assertion,
+                                                             decr_text,
+                                                             tmp_ass.issuer)
+                        if tmp_ass.advice.assertion:
+                            tmp_ass.advice.assertion.extend(advice_res)
+                        else:
+                            tmp_ass.advice.assertion = advice_res
+                        if len(advice_res) > 0:
+                            tmp_ass.advice.encrypted_assertion = []
+            self.response.assertion = resp.assertion
+            for assertion in _enc_assertions:
+                if not self._assertion(assertion, True):
+                    return False
+                else:
+                    self.assertions.append(assertion)
+
+            self.xmlstr = decr_text
+            if len(_enc_assertions) > 0:
+                self.response.encrypted_assertion = []
+
+        if self.response.assertion:
+            for assertion in self.response.assertion:
+                self.assertions.append(assertion)
+
+        if self.assertions and len(self.assertions) > 0:
+            self.assertion = self.assertions[0]
+
+        if self.context == "AuthnReq" or self.context == "AttrQuery":
+            self.ava = self.get_identity()
+            logger.debug("--- AVA: %s" % (self.ava,))
+
+        return True
+
+    def verify(self, keys=None):
         """ Verify that the assertion is syntactically correct and
-        the signature is correct if present."""
-        
+        the signature is correct if present.
+        :param key_file: If not the default key file should be used this is it.
+        """
+
         try:
-            self._verify()
-        except AssertionError:
+            res = self._verify()
+        except AssertionError as err:
+            logger.error("Verification error on the response: %s" % err)
             raise
+        else:
+            if res is None:
+                return None
 
         if not isinstance(self.response, samlp.Response):
             return self
 
-        if self.parse_assertion():
+        if self.parse_assertion(keys):
             return self
         else:
             logger.error("Could not parse the assertion")
             return None
-        
+
     def session_id(self):
-        """ Returns the SessionID of the response """ 
+        """ Returns the SessionID of the response """
         return self.response.in_response_to
-    
+
     def id(self):
         """ Return the ID of the response """
         return self.response.id
-    
+
     def authn_info(self):
         res = []
         for astat in self.assertion.authn_statement:
@@ -841,7 +1011,7 @@ class AuthnResponse(StatusResponse):
         return res
 
     def session_info(self):
-        """ Returns a predefined set of information gleened from the 
+        """ Returns a predefined set of information gleened from the
         response.
         :returns: Dictionary with information
         """
@@ -858,8 +1028,10 @@ class AuthnResponse(StatusResponse):
             return {"ava": self.ava, "name_id": self.name_id,
                     "came_from": self.came_from, "issuer": self.issuer(),
                     "not_on_or_after": nooa, "authn_info": self.authn_info()}
-    
+
     def __str__(self):
+        if not isinstance(self.xmlstr, six.string_types):
+            return "%s" % self.xmlstr.decode("utf-8")
         return "%s" % self.xmlstr
 
     def verify_attesting_entity(self, address):
@@ -892,7 +1064,6 @@ class AuthnQueryResponse(AuthnResponse):
 
     def __init__(self, sec_context, attribute_converters, entity_id,
                  return_addrs=None, timeslack=0, asynchop=False, test=False):
-
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
                                asynchop=asynchop, test=test)
@@ -910,7 +1081,6 @@ class AttributeResponse(AuthnResponse):
 
     def __init__(self, sec_context, attribute_converters, entity_id,
                  return_addrs=None, timeslack=0, asynchop=False, test=False):
-
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
                                asynchop=asynchop, test=test)
@@ -941,7 +1111,6 @@ class ArtifactResponse(AuthnResponse):
 
     def __init__(self, sec_context, attribute_converters, entity_id,
                  return_addrs=None, timeslack=0, asynchop=False, test=False):
-
         AuthnResponse.__init__(self, sec_context, attribute_converters,
                                entity_id, return_addrs, timeslack=timeslack,
                                asynchop=asynchop, test=test)
@@ -953,14 +1122,15 @@ class ArtifactResponse(AuthnResponse):
 
 def response_factory(xmlstr, conf, return_addrs=None, outstanding_queries=None,
                      timeslack=0, decode=True, request_id=0, origxml=None,
-                     asynchop=True, allow_unsolicited=False, want_assertions_signed=False):
+                     asynchop=True, allow_unsolicited=False,
+                     want_assertions_signed=False):
     sec_context = security_context(conf)
     if not timeslack:
         try:
             timeslack = int(conf.accepted_time_diff)
         except TypeError:
             timeslack = 0
-            
+
     attribute_converters = conf.attribute_converters
     entity_id = conf.entityid
     extension_schema = conf.extension_schema
@@ -985,9 +1155,10 @@ def response_factory(xmlstr, conf, return_addrs=None, outstanding_queries=None,
                                     asynchop=asynchop)
         logoutresp.update(response)
         return logoutresp
-        
+
     return response
 
+
 # ===========================================================================
 # A class of it's own
 
@@ -1001,6 +1172,7 @@ class AssertionIDResponse(object):
         self.sec = sec_context
         self.timeslack = timeslack
         self.xmlstr = ""
+        self.origxml = ""
         self.name_id = ""
         self.response = None
         self.not_signed = False
@@ -1013,6 +1185,7 @@ class AssertionIDResponse(object):
         # own copy
         self.xmlstr = xmldata[:]
         logger.debug("xmlstr: %s" % (self.xmlstr,))
+        self.origxml = origxml
 
         try:
             self.response = self.signature_check(xmldata, origdoc=origxml)
@@ -1021,18 +1194,18 @@ class AssertionIDResponse(object):
             raise
         except SignatureError:
             raise
-        except Exception, excp:
+        except Exception as excp:
             logger.exception("EXCEPTION: %s", excp)
             raise
 
-        #print "<", self.response
+        #print("<", self.response)
 
         return self._postamble()
 
-    def verify(self):
+    def verify(self, keys=None):
         try:
             valid_instance(self.response)
-        except NotValid, exc:
+        except NotValid as exc:
             logger.error("Not valid response: %s" % exc.args[0])
             raise
         return self
