@@ -5,10 +5,9 @@
 #
 
 import saml2
-from saml2 import SamlBase
-
-import xmldsig as ds
 from saml2 import saml
+from saml2 import SamlBase
+from saml2 import xmldsig as ds
 
 NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:protocol'
 
@@ -1833,4 +1832,4 @@ def any_response_from_string(xmlstr):
 
     if not resp:
         raise Exception("Unknown response type")
-    return resp
\ No newline at end of file
+    return resp
