@@ -6,13 +6,13 @@ __author__ = 'rolandh'
 def test_eptid():
     edb = Eptid("secret")
     e1 = edb.get("idp_entity_id", "sp_entity_id", "user_id", "some other data")
-    print e1
+    print(e1)
     assert e1.startswith("idp_entity_id!sp_entity_id!")
     e2 = edb.get("idp_entity_id", "sp_entity_id", "user_id", "some other data")
     assert e1 == e2
 
     e3 = edb.get("idp_entity_id", "sp_entity_id", "user_2", "some other data")
-    print e3
+    print(e3)
     assert e1 != e3
 
     e4 = edb.get("idp_entity_id", "sp_entity_id2", "user_id", "some other data")
@@ -23,13 +23,13 @@ def test_eptid():
 def test_eptid_shelve():
     edb = EptidShelve("secret", "eptid.db")
     e1 = edb.get("idp_entity_id", "sp_entity_id", "user_id", "some other data")
-    print e1
+    print(e1)
     assert e1.startswith("idp_entity_id!sp_entity_id!")
     e2 = edb.get("idp_entity_id", "sp_entity_id", "user_id", "some other data")
     assert e1 == e2
 
     e3 = edb.get("idp_entity_id", "sp_entity_id", "user_2", "some other data")
-    print e3
+    print(e3)
     assert e1 != e3
 
     e4 = edb.get("idp_entity_id", "sp_entity_id2", "user_id", "some other data")
@@ -38,4 +38,4 @@ def test_eptid_shelve():
 
 
 if __name__ == "__main__":
-    test_eptid_shelve()
\ No newline at end of file
+    test_eptid_shelve()
